#include <cstdlib>
#include <iostream>
#include "imagemono.h"
#include <SDL/SDL.h>
#include "SDL_flo.h"

/***********************************************
 * Code source du cours sur le dbruitage aux quations aux drives partielles
 *
 * Auteur : HUMBERT Florent
 */

/*
 * Type abstrait de donne G_couleur
 */


G_couleur::G_couleur()
{
 r = (Uint8) 0;
 b = (Uint8) 0;
 g = (Uint8) 0;
}

G_couleur::G_couleur(Uint8 rb, Uint8 gb, Uint8 bb)
{
  r = rb;
  g = gb;
  b = bb;
}


void G_couleur::Ecrire(Uint8 rb, Uint8 gb, Uint8 bb)
{
  r = rb;
  g = gb;
  b = bb;
}

G_surface::G_surface(int x, int y)
{
   SDL_Surface* destination;
   destination = SDL_CreateRGBSurface(SDL_HWSURFACE, x, y, 24,
			0, 0, 0, 0);
   surface = destination;
}

G_surface::~G_surface()
{
  SDL_FreeSurface(surface);
}

void G_surface::Vider()
{
  SDL_FreeSurface(surface);

}

int G_surface::Hauteur()
{
 return surface->h;
}

int G_surface::Largeur()
{
 return surface->w;
}


Uint32 G_surface::LireAux(int x, int y)
{
  int bpp = surface->format->BytesPerPixel;

  Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  switch(bpp)
  {
             case 1:
                  return *p;
             case 2:
                  return *(Uint16 *)p;
             case 3:
                 if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
                     return p[0] << 16 | p[1] << 8 | p[2];
                 else
                     return p[0] | p[1] << 8 | p[2] << 16;
             case 4:


                  return *(Uint32 *)p;
             default:
                  return 0;
  }

}

G_couleur G_surface::Lire(int x, int y)
{
 G_couleur couleur;
 if (x<0 | y<0 | x>=(surface->w) | y>=(surface->h))
 {
   (couleur.Ecrire(0, 0, 0));
   return couleur;
 }
 SDL_GetRGB(LireAux(x, y), surface->format,
                                      (Uint8*)   &couleur.r, (Uint8*) &couleur.g, (Uint8*) &couleur.b);
 return couleur;
}

void G_surface::EcrireAux(int x, int y, Uint32 pixel)
{
    int bpp = surface->format->BytesPerPixel;
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;


    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:

        *(Uint32 *)p = pixel;
        break;
    }

}


void G_surface::Ecrire(int x, int y, G_couleur* couleur)
{
 EcrireAux(x, y,
    SDL_MapRGB(surface->format, couleur->r, couleur->g, couleur->b));
}



G_surface* ImageMono2G_surface(ImageMono* im)
{
  G_couleur couleur;
  int tcou;
  G_surface* gsurf = new G_surface(im->Largeur(), im->Hauteur());

  for(int j=0; j<im->Hauteur(); j++)
   for(int i=0; i<im->Largeur(); i++)
   {
        tcou = (int) min(max((*im)(i,j), 0.0), 255.0);
        couleur.Ecrire(tcou, tcou, tcou);
        gsurf->Ecrire(i, j, &couleur);
   }
 return gsurf;
}

G_surface* ImageMono2G_surfaceAll(ImageMono* im)
{
  G_couleur couleur;
  int tcou;
  double maxi;
  G_surface* gsurf = new G_surface(im->Largeur(), im->Hauteur());

  maxi = (*im)(0,0);


  for(int j=0; j<im->Hauteur(); j++)
   for(int i=0; i<im->Largeur(); i++)
  {
    if((*im)(i, j) > maxi)
      maxi = (*im)(i,j);
  }

  for(int j=0; j<im->Hauteur(); j++)
   for(int i=0; i<im->Largeur(); i++)
   {
        tcou = (int) min(max((*im)(i,j)/maxi * 255.0, 0.0), 255.0);
        couleur.Ecrire(tcou, tcou, tcou);
        gsurf->Ecrire(i, j, &couleur);
   }
 return gsurf;
}


 G_fenetre::G_fenetre(int x, int y)
{
  SDL_Init(SDL_INIT_VIDEO);
  fond = SDL_SetVideoMode(x,y,  32,SDL_HWSURFACE|SDL_DOUBLEBUF);
}

void G_fenetre::Afficher()
{
 SDL_Flip(fond);
}

void G_fenetre::EcrireG_surface(G_surface* surface)
{
 SDL_BlitSurface(surface->surface, NULL, fond, NULL);
}

void G_fenetre::EcrireG_surfaceDec(G_surface* surface)
{
 SDL_Rect rect;
 rect.x = 256;
 rect.y = 0;
 rect.w = 256;
 rect.h = 256;

 SDL_BlitSurface(surface->surface, NULL, fond, &rect);
}

G_fenetre::~G_fenetre()
{
 SDL_FreeSurface(fond);
 SDL_Quit();
}

void G_fenetre::Attendre()
{
 SDL_Event evenement;
  //_Complex complexe = _Complex_I;
 int done = 0;

 while(!done)
    {
        //Traiter les evenements
	while(SDL_PollEvent(&evenement))
	{
		switch(evenement.type)
		{
		case SDL_QUIT:
			done=1;
			break;
   	        case SDL_KEYUP:
			if(evenement.key.keysym.sym==SDLK_q)
				done=1;
			break;
		default:
			break;
		}
	}
  }

}

void G_fenetre::Sauver(char* s)
{
 SDL_SaveBMP(fond, s);
}
