#include <iostream>
#include <cstdlib>
#include <fstream>
#include <cmath>
//#include <math>

#include "imagemono.h"
#define EPSILON 0.1

/***********************************************
 * Code source du cours sur le dbruitage aux quations aux drives partielles
 *
 * Auteur : HUMBERT Florent
 */


using namespace std;


ImageMono* debruitageTikhonov(ImageMono& source, int nbiter, double alpha, double dt)
{
 ImageMono& dest = source.Clone();


 int n;
 int i,j;
 ImageMonoType dxx;
 ImageMonoType dyy;

 for(n=0; n<nbiter;n++)
 {
  for(j=0;j<source.Hauteur();j++)
   for(i=0;i<source.Largeur();i++)
   {
      dxx = dest.Dxx(i,j);
      dyy = dest.Dyy(i,j);
      dest(i, j) =
          dt * (alpha * (dxx + dyy)
           //  -2* (dest->Lire(i,j) - source->Lire(i,j))
         )+ dest(i, j);
   }

 }

 return &dest;
}

ImageMono* debruitageMCF(ImageMono& source, int nbiter, double alpha, double dt)
{
 ImageMono& dest = source.Clone();


 int n;
 int i,j;
 //ImageMonoType dnn;
 ImageMonoType dnt;
 //double cnt;
 //double cn;
 double grad;

 for(n=0; n<nbiter;n++)
 {
  for(j=0;j<source.Hauteur();j++)
   for(i=0;i<source.Largeur();i++)
   {
      grad = dest.NGrad(i,j);
      if(grad> EPSILON)
      {
       dnt = dest.Dntnt(i,j);
       dest(i, j)=
          dt * alpha * (dnt) + dest(i, j);
      }
   }

 }

 return &dest;

}



ImageMono* debruitageHypersurface(ImageMono& source, int nbiter, double alpha, double dt)
{
 ImageMono& dest = source.Clone();
 ImageMono& tempo = source.Clone();


 int n;
 int i,j;
 ImageMonoType dnn;
 ImageMonoType dnt;
 double cnt;
 double cn;
 double grad;
 double opttempo;
 double alphaproduit;
 for(n=0; n<nbiter;n++)
 {
  if(n%10 == 0)
    std::cerr<<100. * (float) n / (float) nbiter<<"% ralis"<<std::endl;
  for(j=0;j<source.Hauteur();j++)
   for(i=0;i<source.Largeur();i++)
   {
      grad = dest.NGrad(i,j);
      if(grad> EPSILON)
      {
       opttempo = 1 + dest.NGradCarre(i,j);
       dnn = dest.Dnn(i,j);
       dnt = dest.Dntnt(i,j);
       cnt = 1.0 / (2.0 * sqrt(opttempo));
       cn = 1.0 / (2.0 * opttempo * sqrt(opttempo));
       alphaproduit =alpha * ((cn * dnn) + (cnt * dnt));
     tempo(i, j)=
          dt * (alpha * ((cn * dnn) + (cnt * dnt))
              + 2*(source(i,j))
          )
         + (-2*dt +1) * dest(i, j)
         ;
      }
   }
  tempo.Copie(&dest);
 }
 delete &tempo;

 return &dest;

}


ImageMono* debruitageVariation(ImageMono& source, int nbiter, double alpha, double dt)
{
 ImageMono& dest = source.Clone();
 ImageMono& tempo = source.Clone();

 int n;
 int i,j;
 ImageMonoType dnt;
 double cnt;
 double grad;

 for(n=0; n<nbiter;n++)
 {
  for(j=0;j<source.Hauteur();j++)
   for(i=0;i<source.Largeur();i++)
   {
      grad = dest.NGrad(i,j);
      if(grad> EPSILON)
      {
       dnt = dest.Dntnt(i,j);
       cnt = 1.0 / (2.0 * grad);
     tempo(i, j)=
          dt * (alpha * ((cnt * dnt))
              - 2*(dest(i,j) - source(i,j))
          )
         + dest(i, j);
      }
   }
  tempo.Copie(&dest);
 }
 delete &tempo;

 return &dest;

}
