#include <iostream>
#include <cstdlib>
#include <fstream>
#include <cmath>
//#include <math>

#include "imagemono.h"

/***********************************************
 * Code source du cours sur le dbruitage aux quations aux drives partielles
 *
 * Auteur : HUMBERT Florent
 */


using namespace std;


ImageMono::ImageMono()
{
 hauteur = 0;
 largeur = 0;
 matrice = NULL;
}

ImageMono::ImageMono(int x, int y)
{
 matrice = new double[x *y];
 hauteur = x;
 largeur = y;
}

ImageMono& ImageMono::Clone()
{
 ImageMono* dest = new ImageMono;
 dest->matrice = new double[largeur * hauteur];
 dest->largeur = largeur;
 dest->hauteur = hauteur;
 for(int i=0; i<largeur * hauteur; i++)
  dest->matrice[i] = matrice[i];

 return *dest;
}

ImageMono::~ImageMono()
{
 delete matrice;
}

void ImageMono::Copie(ImageMono* dest)
{
 for(int i=0; i<largeur * hauteur; i++)
  dest->matrice[i] = matrice[i];
}

int ImageMono::Hauteur()
{
 return hauteur;
}

int ImageMono::Largeur()
{
 return largeur;
}

inline ImageMonoType& ImageMono::operator() (int x, int y)
{
 if(x>=0 && x<largeur && y<hauteur && y>=0)
   return matrice[x + y * largeur];
 else
 {
   if(x<0)
      return (*this)(0,y);
   else
   {
    if(x>=largeur)
      return (*this)(largeur-1, y);
    else
    {
      if(y<0)
        return (*this)(x, 0);
      else
        return (*this)(x, hauteur-1);
    }
   }

 }
}

inline ImageMonoType ImageMono::operator() (int x, int y) const
{
 if(x>=0 && x<largeur && y<hauteur && y>=0)
   return matrice[x + y * largeur];
 else
 {
   if(x<0)
      return (*this)(0,y);
   else
   {
    if(x>=largeur)
      return (*this)(largeur-1, y);
    else
    {
      if(y<0)
        return (*this)(x, 0);
      else
        return (*this)(x, hauteur-1);
    }
   }

 }
}


void ImageMono::ChargerPPM(char * fileName)
{
   int maxIntensity;

   ifstream filePGM(fileName);


   char header[10];
   filePGM.getline(header, 10);

   filePGM >> largeur >> hauteur >> maxIntensity;


   matrice = new double[largeur * hauteur];
   for(int i=0;i<largeur * hauteur;i++)
      filePGM >> matrice[i];
   filePGM.close();
}

ImageMonoType min(ImageMonoType a, ImageMonoType b)
{
 if (a<b)
  return a;
 else
  return b;
}


ImageMonoType max(ImageMonoType a, ImageMonoType b)
{
 if (a<b)
  return b;
 else
  return a;
}


void ImageMono::SauvegarderPPM(char *fileName)
{
   ofstream filePGM(fileName);

   filePGM << "P2" << endl << largeur << " " << hauteur << endl << "255 " << endl;

   for(int i=0;i<largeur* hauteur;i++)
	 filePGM << (int) max(min(matrice[i], 255.0), 0.0) << " ";
   filePGM.close();
}

/*
ImageMono& ImageMono::operator=(const ImageMono& source)
{
 if(largeur != source.largeur || hauteur != source.hauteur)
 {
   std::cerr<<"Problme";
 }

 for(int i=0; i<largeur * hauteur; i++)
  matrice[i] = source.matrice[i];

 return (*this);
}*/

//diffrentiel

ImageMonoType ImageMono::Dx(int x, int y)
{
 return (((*this)(x+1, y)-(*this)(x-1, y) )/2.0);
}

ImageMonoType ImageMono::Dy(int x, int y)
{
 return (((*this)(x, y+1)-(*this)(x, y-1))/2.0);
}

ImageMonoType ImageMono::Dxx(int x, int y)
{
 return ((*this)(x-1, y) + (*this)(x+1, y) - 2.0 * (*this)(x,y));
}

ImageMonoType ImageMono::Dyy(int x, int y)
{
 return ((*this)(x, y-1) + (*this)(x, y+1) - 2.0 * (*this)(x,y));
}

ImageMonoType ImageMono::Dxy(int x, int y)
{
 return 0.25 * ((*this)(x+1, y+1) + (*this)(x-1, y-1) - (*this)(x-1, y+1) - (*this)(x+1, y-1));
 //return (Lire(x+1, y+1)-(*this)(x+1, y) - (*this)(x, y+1) + (*this)(x,y));
}

ImageMonoType ImageMono::NGrad(int x, int y)
{
 return sqrt(Dx(x, y) *Dx(x,y) + Dy(x,y) * Dy(x,y));
}

ImageMonoType ImageMono::NGradCarre(int x, int y)
{
 return (Dx(x, y) *Dx(x,y) + Dy(x,y) * Dy(x,y));
}

/* pour passer  l'approximation de Dnn et de Dntnt de manire classique, il
suffit de dcommenter la ligne suivante o de compiler avec -D_SIMPLE
*/
//#define _SIMPLE
#ifdef _SIMPLE
ImageMonoType ImageMono::Dnn(int x, int y)
{
  ImageMonoType temp;
  temp = Dx(x, y) * Dx(x, y) * Dxx(x, y) + Dy(x,y) * Dy(x,y) * Dyy(x,y)
    + 2.0 * Dx(x,y) * Dy(x,y) * Dxy(x,y);
  return (temp/NGradCarre(x,y));
}

ImageMonoType ImageMono::Dntnt(int x, int y)
{
  ImageMonoType temp;
  temp = Dx(x, y) * Dx(x, y) * Dxx(x, y) + Dy(x,y) * Dy(x,y) * Dyy(x,y)
    - 2.0 * Dx(x,y) * Dy(x,y) * Dxy(x,y);
  return (temp/NGradCarre(x,y));
}

#else

/*Mthode plus performantes du calcul des drives directionnelles*/
ImageMonoType ImageMono::Dntnt(int x, int y)
{
  double fx   = Dx(x,y);
  double fy   = Dy(x,y);

  double fx_grad_carre = (fx * fx)/ (NGradCarre(x,y));
  double fy_grad_carre = (fy * fy) / (NGradCarre(x,y));
  double fxy_grad_carre = (fx * fy) / (NGradCarre(x,y));


  double lambda0 = 0.5;
  double lambda1 = 2.0*lambda0 - fx_grad_carre;
  double lambda2 = 2.0*lambda0 - fy_grad_carre;
  double lambda3 = 0.5 * (-fxy_grad_carre + 1.0 - (2.0 * lambda0));
  double lambda4 = 0.5 * (1.0 - (2.0 * lambda0) + fxy_grad_carre);

  return   ((   lambda4 * (*this)(x-1,y-1)
              + lambda2 * (*this)(x  ,y-1)
              + lambda3 * (*this)(x+1,y-1)
              + lambda1 * (*this)(x-1,y)
          - 4 * lambda0 * (*this)(x  ,y)
              + lambda1 * (*this)(x+1,y)
              + lambda3 * (*this)(x-1,y+1)
              + lambda2 * (*this)(x  ,y+1)
              + lambda4 * (*this)(x+1,y+1)));
}

ImageMonoType ImageMono::Dnn(int x, int y)
{
  double fx   = Dx(x,y);
  double fy   = Dy(x,y);

  double fx_grad_carre = (fx * fx)/ (NGradCarre(x,y));
  double fy_grad_carre = (fy * fy) / (NGradCarre(x,y));
  double fxy_grad_carre = (fx * fy) / (NGradCarre(x,y));

  double lambda0 = 0.5;
  double lambda1 = 2.0*lambda0 - fy_grad_carre;
  double lambda2 = 2.0*lambda0 - fx_grad_carre;
  double lambda3 = 0.5 * (fxy_grad_carre + 1.0 - (2.0 * lambda0));
  double lambda4 = 0.5 * (1.0 - (2.0 * lambda0) - fxy_grad_carre);

  return   ((   lambda4 * (*this)(x-1,y-1)
              + lambda2 * (*this)(x  ,y-1)
              + lambda3 * (*this)(x+1,y-1)
              + lambda1 * (*this)(x-1,y)
          - 4 * lambda0 * (*this)(x  ,y)
              + lambda1 * (*this)(x+1,y)
              + lambda3 * (*this)(x-1,y+1)
              + lambda2 * (*this)(x  ,y+1)
              + lambda4 * (*this)(x+1,y+1)));
}

#endif


